@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: battstat_applet.c,v 1.119 2006/01/04 03:07:17 ryanl Exp $
+ $Id: battstat_applet.c 10289 2007-07-17 06:20:57Z jhaitsma $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,9 @@
 
 #include <sys/types.h>
 #include <sys/wait.h>
+#ifdef HAVE_ERR_H
 #include <err.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -57,6 +59,7 @@
 
 #define GCONF_PATH ""
 
+static gboolean check_for_updates (gpointer data);
 static void about_cb( BonoboUIComponent *, ProgressData *, const char * );
 static void help_cb( BonoboUIComponent *, ProgressData *, const char * );
 
@@ -266,6 +269,34 @@ allocate_battery_colours( void )
   }
 }
 
+/* Our backends may be either event driven or poll-based.
+ * If they are event driven then we know this the first time we
+ * receive an event.
+ */
+static gboolean event_driven = FALSE;
+static GSList *instances;
+
+static void
+status_change_callback (void)
+{
+  GSList *instance;
+
+  for (instance = instances; instance; instance = instance->next)
+  {
+    ProgressData *battstat = instance->data;
+
+    if (battstat->timeout_id)
+    {
+      g_source_remove (battstat->timeout_id);
+      battstat->timeout_id = 0;
+    }
+
+    check_for_updates (battstat);
+  }
+
+  event_driven = TRUE;
+}
+
 /* The following two functions keep track of how many instances of the applet
    are currently running.  When the first instance is started, some global
    initialisation is done.  When the last instance exits, cleanup occurs.
@@ -275,28 +306,34 @@ allocate_battery_colours( void )
    as the process quits immediately when the last applet is removed (which
    it does.)
 */
-static int instances;
-
 static const char *
-static_global_initialisation( int no_hal )
+static_global_initialisation (int no_hal, ProgressData *battstat)
 {
+  gboolean first_time;
   const char *err;
 
-  if( instances++ )
+  first_time = !instances;
+
+  instances = g_slist_prepend (instances, battstat);
+
+  if (!first_time)
     return NULL;
 
   allocate_battery_colours();
   initialise_global_pixmaps();
   glade_init();
-  err = power_management_initialise( no_hal );
+  err = power_management_initialise (no_hal, status_change_callback);
 
   return err;
 }
 
 static void
-static_global_teardown(void)
+static_global_teardown (ProgressData *battstat)
 {
-  if( --instances )
+  instances = g_slist_remove (instances, battstat);
+
+  /* remaining instances... */
+  if (instances)
     return;
 
   /* instances == 0 */
@@ -386,16 +423,37 @@ static gboolean
 battery_full_notify (GtkWidget *applet)
 {
 #ifdef HAVE_LIBNOTIFY
+	GError *error = NULL;
+	GdkPixbuf *icon;
+	gboolean result;
 	
 	if (!notify_is_initted () && !notify_init (_("Battery Monitor")))
 		return FALSE;
 
-	NotifyNotification *n = notify_notification_new (_("Your battery is now fully recharged"), "", "gnome-dev-battery", applet);
+  	icon = gtk_icon_theme_load_icon (
+			gtk_icon_theme_get_default (),
+			"battery",
+			48,
+			GTK_ICON_LOOKUP_USE_BUILTIN,
+			NULL);
+	
+	NotifyNotification *n = notify_notification_new (_("Your battery is now fully recharged"), "", /* "battery" */ NULL, applet);
 
-	if(!notify_notification_show_and_forget (n, NULL))
-	   return FALSE;
+	/* XXX: it would be nice to pass this as a named icon */
+	notify_notification_set_icon_from_pixbuf (n, icon);
+	gdk_pixbuf_unref (icon);
 
-	return TRUE;
+	result = notify_notification_show (n, &error);
+	
+	if (error)
+	{
+	   g_warning (error->message);
+	   g_error_free (error);
+	}
+
+	g_object_unref (G_OBJECT (n));
+
+	return result;
 #else
 	return FALSE;
 #endif
@@ -430,7 +488,7 @@ battery_full_dialog (GtkWidget *applet)
   hbox = gtk_hbox_new (FALSE, 6);
   pixbuf = gtk_icon_theme_load_icon (
 		gtk_icon_theme_get_default (),
-		"gnome-dev-battery",
+		"battery",
 		48,
 		GTK_ICON_LOOKUP_USE_BUILTIN,
 		NULL);
@@ -513,13 +571,22 @@ battery_low_update_text( ProgressData *battstat, BatteryStatus *info )
     gtk_widget_set_size_request( GTK_WIDGET( battstat->battery_low_label ),
                                  size.width, size.height );
 
-  remaining = g_strdup_printf( ngettext(
+  if (info->minutes < 0 && !info->on_ac_power)
+  {
+	  /* we don't know the remaining time */
+	  remaining = g_strdup_printf (_("You have %d%% of your total battery "
+				  	 "capacity remaining."), info->percent);
+  }
+  else
+  {
+	  remaining = g_strdup_printf( ngettext(
                                  "You have %d minute of battery power "
 				   "remaining (%d%% of the total capacity).",
                                  "You have %d minutes of battery power "
 				   "remaining (%d%% of the total capacity).",
                                  info->minutes ),
                                info->minutes,info->percent );
+  }
 
   if( is_suspend_unavailable() )
     /* TRANSLATORS: this is a list, it is left as a single string
@@ -583,7 +650,7 @@ battery_low_dialog( ProgressData *battery, BatteryStatus *info )
   hbox = gtk_hbox_new (FALSE, 6);
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
   pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-		 "gnome-dev-battery",
+		 "battery",
 		 48,
 		 GTK_ICON_LOOKUP_USE_BUILTIN,
 		 NULL);
@@ -608,6 +675,8 @@ battery_low_dialog( ProgressData *battery, BatteryStatus *info )
 
   battery_low_update_text( battery, info );
 
+  gtk_window_set_position (GTK_WINDOW (battery->battery_low_dialog),
+		  GTK_WIN_POS_CENTER);
   gtk_widget_show_all (battery->battery_low_dialog);
 }
 
@@ -642,7 +711,7 @@ update_tooltip( ProgressData *battstat, BatteryStatus *info )
                                  _("Battery status unknown"));
   }
 
-  gtk_tooltips_set_tip (battstat->tip, battstat->applet, tiptext, NULL);
+  gtk_widget_set_tooltip_text (battstat->applet, tiptext);
   g_free (tiptext);
 }
 
@@ -940,7 +1009,7 @@ possibly_update_status_icon( ProgressData *battstat, BatteryStatus *info )
 /* Gets called as a gtk_timeout once per second.  Checks for updates and
    makes any changes as appropriate.
  */
-static gint
+static gboolean
 check_for_updates( gpointer data )
 {
   ProgressData *battstat = data;
@@ -952,6 +1021,30 @@ check_for_updates( gpointer data )
   if( (err = power_management_getinfo( &info )) )
     battstat_error_dialog( battstat->applet, err );
 
+  if (!event_driven)
+  {
+    int timeout;
+
+    /* if on AC and not even driven scale back the polls to once every 10 */
+    if (info.on_ac_power)
+      timeout = 10000;
+    else
+      timeout = 1000;
+
+    if (timeout != battstat->timeout)
+    {
+      battstat->timeout = timeout;
+
+      if (battstat->timeout_id)
+        g_source_remove (battstat->timeout_id);
+
+      battstat->timeout_id = g_timeout_add (battstat->timeout,
+                                            check_for_updates,
+                                            battstat);
+    }
+  }
+
+
   possibly_update_status_icon( battstat, &info );
 
   if (!info.on_ac_power &&
@@ -1058,16 +1151,14 @@ destroy_applet( GtkWidget *widget, ProgressData *battstat )
 {
   if (DEBUG) g_print("destroy_applet()\n");
 
-  if (battstat->about_dialog)
-    gtk_widget_destroy (battstat->about_dialog);
-
   if (battstat->prop_win)
     gtk_widget_destroy (GTK_WIDGET (battstat->prop_win));
 
   if( battstat->battery_low_dialog )
     battery_low_dialog_destroy( battstat );
 
-  gtk_timeout_remove( battstat->pixtimer );
+  if (battstat->timeout_id)
+    g_source_remove (battstat->timeout_id);
 
   if( battstat->pixgc )
     g_object_unref( G_OBJECT(battstat->pixgc) );
@@ -1075,11 +1166,10 @@ destroy_applet( GtkWidget *widget, ProgressData *battstat )
   g_object_unref( G_OBJECT(battstat->status) );
   g_object_unref( G_OBJECT(battstat->percent) );
   g_object_unref( G_OBJECT(battstat->battery) );
-  g_object_unref( G_OBJECT(battstat->tip) );
 
   g_free( battstat );
 
-  static_global_teardown();
+  static_global_teardown (battstat);
 }
 
 /* Common function invoked by the 'Help' context menu item and the 'Help'
@@ -1137,22 +1227,25 @@ about_cb( BonoboUIComponent *uic, ProgressData *battstat, const char *verb )
     NULL
   };
 
-  if( power_management_using_hal() )
-    authors[4] = "Ryan Lortie <desrt@desrt.ca> \xe2\x98\x86";
+  char *comments = g_strdup_printf ("%s\n\n%s",
+		  _("This utility shows the status of your laptop battery."),
+		  power_management_using_hal () ?
+		  	/* true */ _("HAL backend enabled.") :
+			/* false */ _("Legacy (non-HAL) backend enabled."));
 
   gtk_show_about_dialog( NULL,
-    "name",                _("Battery Charge Monitor"), 
     "version",             VERSION,
     "copyright",           "\xC2\xA9 2000 The Gnulix Society, "
                            "\xC2\xA9 2002-2005 Free Software Foundation and "
                            "others",
-    "comments",            _("This utility shows the status of your laptop "
-                             "battery."),
+    "comments",            comments,
     "authors",             authors,
     "documenters",         documenters,
     "translator-credits",  _("translator-credits"),
-    "logo-icon-name",      "gnome-dev-battery",
+    "logo-icon-name",      "battery",
     NULL );
+
+  g_free (comments);
 }
 
 /* Rotate text on side panels.  Called on initial startup and when the
@@ -1481,15 +1574,6 @@ create_layout(ProgressData *battstat)
   gtk_container_add (GTK_CONTAINER (battstat->applet), battstat->table);
   gtk_widget_show_all (battstat->applet);
 
-  /* Set up the tooltip widget (which we will fill in later). */
-  battstat->tip = gtk_tooltips_new ();
-  g_object_ref (battstat->tip);
-  gtk_object_sink (GTK_OBJECT (battstat->tip));
-  gtk_tooltips_set_tip (battstat->tip,
-			battstat->applet,
-			"",
-			NULL);
-
   /* Attach all sorts of signals to the applet. */
   g_signal_connect(G_OBJECT(battstat->applet),
                    "destroy",
@@ -1521,12 +1605,9 @@ battstat_applet_fill (PanelApplet *applet)
 
   if (DEBUG) g_print("main()\n");
 
-  no_hal = panel_applet_gconf_get_bool( applet, "no_hal", NULL );
+  g_set_application_name (_("Battery Charge Monitor"));
 
-  if( (err = static_global_initialisation( no_hal )) )
-    battstat_error_dialog( GTK_WIDGET (applet), err );
-
-  gtk_window_set_default_icon_name ("gnome-dev-battery");
+  gtk_window_set_default_icon_name ("battery");
   
   panel_applet_add_preferences (applet, "/schemas/apps/battstat-applet/prefs",
                                 NULL);
@@ -1545,8 +1626,9 @@ battstat_applet_fill (PanelApplet *applet)
   battstat->horizont = TRUE;
   battstat->battery_low_dialog = NULL;
   battstat->battery_low_label = NULL;
-  battstat->about_dialog = NULL;
   battstat->pixgc = NULL;
+  battstat->timeout = -1;
+  battstat->timeout_id = 0;
 
   /* The first received size_allocate event will cause a reconfigure. */
   battstat->height = -1;
@@ -1556,8 +1638,6 @@ battstat_applet_fill (PanelApplet *applet)
   create_layout (battstat);
   setup_text_orientation( battstat );
 
-  battstat->pixtimer = gtk_timeout_add (1000, check_for_updates, battstat);
-
   panel_applet_setup_menu_from_file (PANEL_APPLET (battstat->applet), 
   			             DATADIR,
                                      "GNOME_BattstatApplet.xml",
@@ -1582,6 +1662,11 @@ battstat_applet_fill (PanelApplet *applet)
 	  atk_object_set_description(atk_widget, _("Monitor a laptop's remaining power"));
   }
 
+  no_hal = panel_applet_gconf_get_bool( applet, "no_hal", NULL );
+
+  if ((err = static_global_initialisation (no_hal, battstat)))
+    battstat_error_dialog (GTK_WIDGET (applet), err);
+
   return TRUE;
 }
 
